@@ -12,10 +12,22 @@ PKG_NAME="gnome-initial-setup"
 (test -f $srcdir/configure.ac \
   && test -f $srcdir/gnome-initial-setup.doap) || {
     echo -n "**Error**: Directory "\`$srcdir\'" does not look like the"
-    echo " top-level gnome-settings-daemon directory"
+    echo " top-level gnome-initial-setup directory"
     exit 1
 }
 
+# Fetch submodules if needed
+if test ! -f egg-list-box/COPYING;
+then
+  echo "+ Setting up submodules"
+  git submodule init
+fi
+git submodule update
+
+cd egg-list-box
+sh autogen.sh --no-configure
+cd ..
+
 which gnome-autogen.sh || {
     echo "You need to install gnome-common"
     exit 1
