@@ -3,25 +3,24 @@
  * Copyright (C) 2012 Red Hat
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  *
  * Written by:
  *     Jasper St. Pierre <jstpierre@mecheye.net>
  */
 
-
 #include "config.h"
 
 #include <glib-object.h>
@@ -37,43 +36,35 @@ G_DEFINE_TYPE (GisAssistantGtk, gis_assistant_gtk, GIS_TYPE_ASSISTANT)
 
 struct _GisAssistantGtkPrivate
 {
-  GtkWidget *frame;
+  GtkWidget *notebook;
 };
 
 static void
-set_current_page (GisAssistant *assistant,
-                  GtkWidget    *new_page)
+current_page_changed (GtkNotebook  *notebook,
+                      GtkWidget    *new_page,
+                      gint          new_page_num,
+                      GisAssistant *assistant)
 {
-  GisAssistantGtkPrivate *priv = GIS_ASSISTANT_GTK (assistant)->priv;
-  GtkBin *bin = GTK_BIN (priv->frame);
-
-  GtkWidget *old_page = gtk_bin_get_child (bin);
-
-  if (old_page != NULL)
-    gtk_container_remove (GTK_CONTAINER (bin), old_page);
-
-  if (new_page != NULL)
-    gtk_container_add (GTK_CONTAINER (bin), new_page);
-
-  _gis_assistant_current_page_changed (assistant, new_page);
+  _gis_assistant_current_page_changed (assistant, GIS_PAGE (new_page));
 }
 
 static void
 gis_assistant_gtk_switch_to (GisAssistant *assistant,
-                             GtkWidget    *widget)
+                             GisPage      *page)
 {
-  set_current_page (assistant, widget);
+  GisAssistantGtkPrivate *priv = GIS_ASSISTANT_GTK (assistant)->priv;
+  gint page_num = gtk_notebook_page_num (GTK_NOTEBOOK (priv->notebook),
+                                         GTK_WIDGET (page));
+  gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), page_num);
 }
 
 static void
 gis_assistant_gtk_add_page (GisAssistant *assistant,
-                            GtkWidget    *widget)
+                            GisPage      *page)
 {
   GisAssistantGtkPrivate *priv = GIS_ASSISTANT_GTK (assistant)->priv;
-  GtkWidget *old_child = gtk_bin_get_child (GTK_BIN (priv->frame));
-
-  if (old_child == NULL)
-    set_current_page (assistant, widget);
+  gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook),
+                            GTK_WIDGET (page), NULL);
 }
 
 static void
@@ -81,10 +72,19 @@ gis_assistant_gtk_init (GisAssistantGtk *assistant_gtk)
 {
   GisAssistantGtkPrivate *priv = GET_PRIVATE (assistant_gtk);
   GisAssistant *assistant = GIS_ASSISTANT (assistant_gtk);
+  GtkWidget *frame;
 
   assistant_gtk->priv = priv;
 
-  priv->frame = _gis_assistant_get_frame (assistant);
+  frame = _gis_assistant_get_frame (assistant);
+  priv->notebook = gtk_notebook_new ();
+  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (priv->notebook), FALSE);
+  gtk_container_add (GTK_CONTAINER (frame), priv->notebook);
+
+  gtk_widget_show (priv->notebook);
+
+  g_signal_connect (priv->notebook, "switch-page",
+                    G_CALLBACK (current_page_changed), assistant);
 }
 
 static void
@@ -94,6 +94,6 @@ gis_assistant_gtk_class_init (GisAssistantGtkClass *klass)
 
   g_type_class_add_private (klass, sizeof (GisAssistantGtkPrivate));
 
-  assistant_class->switch_to = gis_assistant_gtk_switch_to;
   assistant_class->add_page = gis_assistant_gtk_add_page;
+  assistant_class->switch_to = gis_assistant_gtk_switch_to;
 }
