@@ -1,11 +1,30 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
-/* GIS_ASSISTANT */
-/* GisAssistant */
+/*
+ * Copyright (C) 2012 Red Hat
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ *
+ * Written by:
+ *     Jasper St. Pierre <jstpierre@mecheye.net>
+ */
 
 #ifndef __GIS_ASSISTANT_H__
 #define __GIS_ASSISTANT_H__
 
-#include <glib-object.h>
+#include "gis-page.h"
 
 G_BEGIN_DECLS
 
@@ -31,41 +50,24 @@ struct _GisAssistantClass
 {
   GtkBoxClass parent_class;
 
-  void (* prepare) (GisAssistant *assistant, GtkWidget *page);
-  void (* next_page) (GisAssistant *assistant, GtkWidget *page);
+  void (* prepare) (GisAssistant *assistant, GisPage *page);
+  void (* next_page) (GisAssistant *assistant, GisPage *page);
 
-  void (* switch_to) (GisAssistant *assistant, GtkWidget *page);
-  void (* add_page) (GisAssistant *assistant, GtkWidget *page);
+  void (* switch_to) (GisAssistant *assistant, GisPage *page);
+  void (* add_page) (GisAssistant *assistant, GisPage *page);
 };
 
 GType gis_assistant_get_type (void);
 
 void      gis_assistant_add_page          (GisAssistant *assistant,
-                                           GtkWidget    *page);
+                                           GisPage      *page);
 
 void      gis_assistant_next_page         (GisAssistant *assistant);
 void      gis_assistant_previous_page     (GisAssistant *assistant);
 void      gis_assistant_destroy_all_pages (GisAssistant *assistant);
-
-void      gis_assistant_set_page_complete (GisAssistant *assistant,
-                                           GtkWidget    *page,
-                                           gboolean      complete);
-gboolean  gis_assistant_get_page_complete (GisAssistant *assistant,
-                                           GtkWidget    *page);
-
-void      gis_assistant_set_page_title    (GisAssistant *assistant,
-                                           GtkWidget    *page,
-                                           gchar        *title);
-gchar *   gis_assistant_get_page_title    (GisAssistant *assistant,
-                                           GtkWidget    *page);
 gchar *   gis_assistant_get_title         (GisAssistant *assistant);
 
-void      gis_assistant_set_use_unicode_buttons (GisAssistant *assistant,
-                                                 GtkWidget    *page,
-                                                 gboolean      use_unicode_buttons);
-
-gboolean  gis_assistant_get_use_unicode_buttons (GisAssistant *assistant,
-                                                 GtkWidget    *page);
+void      gis_assistant_locale_changed    (GisAssistant *assistant);
 
 G_END_DECLS
 
