@@ -56,13 +56,19 @@ enum {
 
 struct _GisLanguagePagePrivate
 {
-  GtkWidget *show_all;
+  GtkWidget *more_item;
   GtkWidget *page;
   GtkWidget *filter_entry;
   GtkWidget *language_list;
   gboolean adding_languages;
 };
 
+/* We keep this outside the private struct, so that it survives
+ * reconstructing the pages. Otherwise, the ... 'snaps back'
+ * whenever you select a different language.
+ */
+static gboolean showing_extra;
+
 #define OBJ(type,name) ((type)gtk_builder_get_object(GIS_PAGE (page)->builder,(name)))
 #define WID(name) OBJ(GtkWidget*,name)
 
@@ -83,6 +89,15 @@ sort_languages (gconstpointer a,
                 gconstpointer b,
                 gpointer      data)
 {
+  GisLanguagePage *page = GIS_LANGUAGE_PAGE (data);
+  GisLanguagePagePrivate *priv = page->priv;
+
+  if (a == priv->more_item)
+    return 1;
+
+  if (b == priv->more_item)
+    return -1;
+
   const char *la = g_object_get_data (G_OBJECT (a), "locale-name");
   const char *lb = g_object_get_data (G_OBJECT (b), "locale-name");
 
@@ -111,6 +126,46 @@ use_language (char *locale_id)
   return g_strdup_printf (use, language);
 }
 
+static GtkWidget *
+language_widget_new (char     *locale_id,
+                     gboolean  is_extra)
+{
+  gchar *locale_name;
+  GtkWidget *widget;
+  gchar *current_locale_id = cc_common_language_get_current_language ();
+
+  locale_name = use_language (locale_id);
+  setlocale (LC_MESSAGES, current_locale_id);
+
+  widget = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
+  gtk_widget_set_halign (widget, GTK_ALIGN_CENTER);
+  gtk_widget_set_margin_top (widget, 10);
+  gtk_widget_set_margin_bottom (widget, 10);
+  gtk_box_pack_start (GTK_BOX (widget), gtk_label_new (locale_name), FALSE, FALSE, 0);
+
+  if (g_strcmp0 (locale_id, current_locale_id) == 0)
+    {
+      gtk_box_pack_start (GTK_BOX (widget),
+                          gtk_image_new_from_icon_name ("object-select-symbolic", GTK_ICON_SIZE_MENU),
+                          FALSE, FALSE, 0);
+    }
+  g_free (current_locale_id);
+
+  g_object_set_data (G_OBJECT (widget), "locale-id", locale_id);
+  g_object_set_data (G_OBJECT (widget), "locale-name", locale_name);
+  g_object_set_data (G_OBJECT (widget), "is-extra", GUINT_TO_POINTER (is_extra));
+
+  return widget;
+}
+
+static GtkWidget *
+more_widget_new (void)
+{
+  GtkWidget *widget = gtk_label_new ("…");
+  gtk_widget_set_tooltip_text (widget, _("More…"));
+  return widget;
+}
+
 static void
 add_languages (GisLanguagePage *page,
                char           **locale_ids,
@@ -123,7 +178,6 @@ add_languages (GisLanguagePage *page,
 
   while (*locale_ids) {
     gchar *locale_id;
-    gchar *locale_name;
     gboolean is_extra;
     GtkWidget *widget;
 
@@ -135,24 +189,18 @@ add_languages (GisLanguagePage *page,
       continue;
 
     is_extra = (g_hash_table_lookup (initial, locale_id) != NULL);
-    locale_name = use_language (locale_id);
 
-    widget = gtk_label_new (locale_name);
-    g_object_set_data (G_OBJECT (widget), "locale-id",
-                       locale_id);
-    g_object_set_data (G_OBJECT (widget), "locale-name",
-                       locale_name);
-    g_object_set_data (G_OBJECT (widget), "is-extra",
-                       GUINT_TO_POINTER (is_extra));
-    gtk_widget_show (widget);
+    widget = language_widget_new (locale_id, is_extra);
 
     gtk_container_add (GTK_CONTAINER (priv->language_list),
                        widget);
-
-    if (strcmp (locale_id, orig_locale_id) == 0)
-      egg_list_box_select_child (EGG_LIST_BOX (priv->language_list), widget);
   }
 
+  gtk_container_add (GTK_CONTAINER (priv->language_list),
+                     priv->more_item);
+
+  gtk_widget_show_all (priv->language_list);
+
   priv->adding_languages = FALSE;
 
   setlocale (LC_MESSAGES, orig_locale_id);
@@ -176,33 +224,40 @@ language_visible (GtkWidget *child,
   GisLanguagePagePrivate *priv = page->priv;
   gchar *locale_name;
   const gchar *filter_contents;
-  gboolean visible = TRUE;
   gboolean is_extra;
 
+  if (child == page->priv->more_item)
+    return !showing_extra;
+
   is_extra = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (child), "is-extra"));
   locale_name = g_object_get_data (G_OBJECT (child), "locale-name");
 
   filter_contents = gtk_entry_get_text (GTK_ENTRY (priv->filter_entry));
   if (*filter_contents && strcasestr (locale_name, filter_contents) == NULL)
-    {
-      visible = FALSE;
-      goto out;
-    }
+    return FALSE;
 
-  if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->show_all)) && !is_extra)
-    {
-      visible = FALSE;
-      goto out;
-    }
+  if (!showing_extra && !is_extra)
+    return FALSE;
 
- out:
-  return visible;
+  return TRUE;
 }
 
 static void
-selection_changed (EggListBox      *box,
-                   GtkWidget       *child,
-                   GisLanguagePage *page)
+show_more (GisLanguagePage *page)
+{
+  GisLanguagePagePrivate *priv = page->priv;
+
+  gtk_widget_show (priv->filter_entry);
+
+  showing_extra = TRUE;
+
+  egg_list_box_refilter (EGG_LIST_BOX (priv->language_list));
+}
+
+static void
+child_activated (EggListBox      *box,
+                 GtkWidget       *child,
+                 GisLanguagePage *page)
 {
   gchar *new_locale_id;
 
@@ -212,22 +267,16 @@ selection_changed (EggListBox      *box,
   if (child == NULL)
     return;
 
+  if (child == page->priv->more_item)
+    {
+      show_more (page);
+      return;
+    }
+
   new_locale_id = g_object_get_data (G_OBJECT (child), "locale-id");
   set_locale_id (page, new_locale_id);
 }
 
-static void
-show_all_toggled (GtkCheckButton  *button,
-                  GisLanguagePage *page)
-{
-  GisLanguagePagePrivate *priv = page->priv;
-
-  gtk_widget_hide (GTK_WIDGET (button));
-  gtk_widget_show (priv->filter_entry);
-
-  egg_list_box_refilter (EGG_LIST_BOX (priv->language_list));
-}
-
 static void
 gis_language_page_constructed (GObject *object)
 {
@@ -238,29 +287,29 @@ gis_language_page_constructed (GObject *object)
 
   gtk_container_add (GTK_CONTAINER (page), WID ("language-page"));
 
-  priv->show_all = WID ("language-show-all");
   priv->filter_entry = WID ("language-filter-entry");
   priv->language_list = WID ("language-list");
+  priv->more_item = more_widget_new ();
+
+  egg_list_box_set_adjustment (EGG_LIST_BOX (priv->language_list),
+                               gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (WID ("language-scrolledwindow"))));
 
   egg_list_box_set_sort_func (EGG_LIST_BOX (priv->language_list),
                               sort_languages, page, NULL);
   egg_list_box_set_filter_func (EGG_LIST_BOX (priv->language_list),
                                 language_visible, page, NULL);
+  egg_list_box_set_selection_mode (EGG_LIST_BOX (priv->language_list),
+                                   GTK_SELECTION_NONE);
   add_all_languages (page);
 
-  g_signal_connect (priv->show_all, "toggled",
-                    G_CALLBACK (show_all_toggled),
-                    page);
-
   g_signal_connect_swapped (priv->filter_entry, "changed",
                             G_CALLBACK (egg_list_box_refilter),
                             priv->language_list);
 
-  g_signal_connect (priv->language_list, "child-selected",
-                    G_CALLBACK (selection_changed), page);
+  g_signal_connect (priv->language_list, "child-activated",
+                    G_CALLBACK (child_activated), page);
 
   gis_page_set_complete (GIS_PAGE (page), TRUE);
-  gis_page_set_use_arrow_buttons (GIS_PAGE (page), TRUE);
   gis_page_set_title (GIS_PAGE (page), _("Welcome"));
 
   gtk_widget_show (GTK_WIDGET (page));
